﻿using System;
using System.Device.Location;
using System.Globalization;
using System.Windows.Data;

namespace CoordinateDisplay
{
	/// <summary>
	/// Converts a GeoCoordinate to Degrees-Minutes-Seconds
	/// </summary>
	public class CoordinateConverter : IValueConverter
	{
		private const double MeanEarthRadius = 6367449; //meters
		private const double MeanEarthCircumference = 2 * Math.PI * MeanEarthRadius; //meters
		public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
		{
			if (value is GeoCoordinate)
			{
				var coord = value as GeoCoordinate;
				if(coord.IsUnknown) return "Unknown";

				double lat = coord.Latitude;
				double lon = coord.Longitude;
				if ((parameter is string) &&
					string.Compare("decimal", parameter as string, StringComparison.OrdinalIgnoreCase) == 0)
				//show as decimal degrees
				{
					var decimalsLat = 7;
					var decimalsLon = 7;
					int val = 4;
					if (coord.HorizontalAccuracy > val * 100) decimalsLat = 5;
					else if (coord.HorizontalAccuracy > val * 10) decimalsLat = 6;
					val = (int)Math.Floor(Math.Cos(lat / 180 * Math.PI) * MeanEarthCircumference / 10000000d);
					if (coord.HorizontalAccuracy > val * 100) decimalsLon = 5;
					else if (coord.HorizontalAccuracy > val * 10) decimalsLon = 6;
					return string.Format("{0}°,{1}°", Math.Round(lat, decimalsLat), Math.Round(lon, decimalsLon));
				}
				else //Show as degrees/minutes/seconds
				{
					char ns = lat < 0 ? 'S' : 'N'; //Southern or Northern hemisphere?
					char ew = lon < 0 ? 'W' : 'E'; //Eastern or Western hemisphere?
					//Make positive
					lon = Math.Abs(lon);
					lat = Math.Abs(lat);
					//Grab the part in front of the decimal
					var majorLong = Math.Floor(lon);
					var majorLat = Math.Floor(lat);
					//the value after the decimal in minutes (*60)
					var minorLong = (lon - majorLong) * 60;
					var minorLat = (lat - majorLat) * 60;
					//Seconds:
					var minutesLong = Math.Floor(minorLong);
					var minutesLat = Math.Floor(minorLat);

					//one digit accuracy on one second equals ~3m or better
					//this changes with the latitude, but this is good enough for now
					int decimals = 1;
					if (coord.HorizontalAccuracy > 30)
						decimals = 0; //With this accuracy we don't need to show sub-second accuracy
					//Seconds:
					var secondsLong = Math.Round((minorLong - minutesLong) * 60, decimals);
					var secondsLat = Math.Round((minorLat - minutesLat) * 60, decimals);
					return string.Format("{0}{1}°{2}'{3}\" {4}{5}°{6}'{7}\"",
						ns, majorLat, minutesLat, secondsLat,
						ew, majorLong, minutesLong, secondsLong);
				}
			}
			return value;
		}

		public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
		{
			throw new NotSupportedException();
		}
	}
}
